/* The MuJS console object implementation. */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "elinks.h"

#include "ecmascript/ecmascript.h"
#include "ecmascript/mujs.h"
#include "ecmascript/mujs/console.h"


#define DEBUG 0

static void
mjs_console_log_common(js_State *J, const char *str, const char *log_filename)
{
#ifdef ECMASCRIPT_DEBUG
	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
#endif
	if (log_filename && get_opt_bool("ecmascript.enable_console_log", NULL) && str)
	{
		FILE *f = fopen(log_filename, "a");

		if (f)
		{
			fprintf(f, "%s\n", str);
			fclose(f);
		}
	}
	js_pushundefined(J);
}

static void
mjs_console_log(js_State *J)
{
#ifdef ECMASCRIPT_DEBUG
	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
#endif
	const char *str = js_tostring(J, 1);

	mjs_console_log_common(J, str, console_log_filename);
}

static void
mjs_console_error(js_State *J)
{
#ifdef ECMASCRIPT_DEBUG
	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
#endif
	const char *str = js_tostring(J, 1);

	mjs_console_log_common(J, str, console_error_filename);
}

static void
mjs_console_toString(js_State *J)
{
#ifdef ECMASCRIPT_DEBUG
	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
#endif
	js_pushstring(J, "[console object]");
}

int
mjs_console_init(js_State *J)
{
	js_newobject(J);
	{
		addmethod(J, "console.log", mjs_console_log, 1);
		addmethod(J, "console.error", mjs_console_error, 1);
		addmethod(J, "console.toString", mjs_console_toString, 0);
	}
	js_defglobal(J, "console", JS_DONTENUM);

	return 0;
}
